/*
    setpanel.c - A user-space program to write to the ASUS DigiMatrix front panel.
    Copyright (c) 2005 Richard Taylor <richard@artaylor.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/io.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <time.h>
#include <alsa/asoundlib.h>
#include "i2c-dev.h"
#include "setpanel.h"
#include "sndctrl.h"
#include "i2cfindsis96x.h"

//BOOL bI2CInitialised = FALSE;

static struct option longopts[] = {
//    { "i2c",         1, 0, 'i' },
    { "radio",       0, 0, 'r' },
    { "control",     1, 0, 'c' },
    { "display",     1, 0, 'd' },
    { "reinit",      1, 0, 'D' },
    { "temp",        0, 0, 't' },
    { "timetemp",    0, 0, 'T' },
    { "update-temp", 0, 0, 'u' },
    { "volume",      0, 0, 'l' },
    { "vol",         0, 0, 'L' },
    { "manual",      0, 0, 'm' },
    { "time",        0, 0, 's' },
    { "version",     0, 0, 'v' },
    { "help",        0, 0, 'h' },
    { NULL,          0, 0,  0  }
};

// "  -i, --i2c <n>              Manually set the I2C Bus\n"
static void help(char *argv0)
{
        printf(
"\n"
"Usage: %s [OPTIONS]\n"
"\n"
"  If you are using this on a system other than the ASUS DigiMatrix,\n"
"  then there is a chance that you may destroy an EEPROM.\n"
"\n"
"  -r, --radio [am | fm | off]  Set the radio to am, fm or off\n"
"\n"
"  -c, --control [seek- | seek+ | fine- | fine+ | ch- | ch+ | prog | cancel | store]\n"
"                             Controls the radio\n"
"                             seek+/- seeks the next station\n"
"                             fine+/- manual increases / decreases frequency\n"
"                             ch+/- selects the stored presets\n"
"                             prog begins storing the current frequency (use ch+/- to choose)\n"
"                             cancel aborts the store\n"
"                             store commits the frequency to the flashing preset\n"
"\n"
"  -d, --display [\"xxxxxx\"]   Set the display to the numbers specified\n"
"                             Available are: [0-9, A, C, F, P]\n"
"                             Limited availability: [M, D, -] (try them :)\n"
"                             A Capital letter in digit[0] will also light up the mode led\n"
"                             A small letter will suppress the led\n"
"                             Anything else will be displayed as blank.\n"
"                             Adding: [am | fm | cd | hd]\n"
"                                     [play | pause]\n"
"                                     [both | middle | bottom]\n"
"                             will activate the relevant LED / dots.\n"
"  e.g.\n"
"  (1) -d 123456 will display 12 3456 \n"
"  (2) -d \"123456 fm pause both\" will display 12 34:56 and also light up the fm and pause leds.\n"
"  N.B. You cannot have multiple mode LEDs lit at the same time (e.g. am + fm, or play + pause).\n"
"\n"
"  -D, --reinit [\"xxxxxx\"]  As above, but re-initilises the display first\n"
"\n"
"  -t, --temp                 Display the current temperature.\n"
"  -T, --timetemp             Display both the current temperature and time.\n"
"  -u, --update-temp          Updates the temperature every 2 seconds.\n"
"  -l, --volume               Displays any changes on the volume control.\n"
"  -L, --vol                  As above, but displays no other text.\n"
"  -m, --manual               Allows you to interactively change a register.\n"
"  -s, --time                 Sets and displays the current local time.\n"
"\n"
"  -v, --version              Print version\n"
"  -h, --help                 This help\n"
"\n", argv0);

  //printf("\n\nFor available I2C busses, use i2cdetect.\nYou are looking for \"SiS96x SMBus adapter at 0xe600\"\n\n");
  exit(1);
}

static int digimatrix_write_byte_data(int file, char reg, char data)
{
    int res;
    // TODO: Implement kernel specific stuff here. 
    res = i2c_smbus_write_byte_data(file, reg, data);
    usleep(I2C_TRANSACTION_DELAY_MS);
    return res;
}

static int digimatrix_write_byte(int file, char data)
{
    int res;
    // TODO: Implement kernel specific stuff here. 
    res = i2c_smbus_write_byte(file, data);
    usleep(I2C_TRANSACTION_DELAY_MS);
    return res;
}

static int digimatrix_read_byte(int file)
{
    int res;
    // TODO: Implement kernel specific stuff here. 
    res = i2c_smbus_read_byte(file);
    usleep(I2C_TRANSACTION_DELAY_MS);
    return res;
}

void digimatrix_Control(int file, CONTROL control) { 
 
    switch (control) { 
        case SEEK_DOWN: 
            digimatrix_write_byte_data(file, 0x27, 0x00); 
            break; 
 
        case SEEK_UP: 
            digimatrix_write_byte_data(file, 0x13, 0x00); 
            break; 
 
        case FINE_TUNE_DOWN: 
            digimatrix_write_byte_data(file, 0x26, 0x00); 
            break; 
 
        case FINE_TUNE_UP: 
            digimatrix_write_byte_data(file, 0x25, 0x00); 
            break; 
 
        case CHANNEL_PREV: 
            digimatrix_write_byte_data(file, 0x11, 0x00); 
            break; 
 
        case CHANNEL_NEXT: 
            digimatrix_write_byte_data(file, 0x10, 0x00); 
            break; 
 
        case STORE_START: 
            digimatrix_write_byte_data(file, 0x15, 0x00); 
            break; 
 
        case STORE_CANCEL: 
            digimatrix_write_byte_data(file, 0x16, 0x00); 
            break; 
        
	case STORE_COMMIT: 
            digimatrix_write_byte_data(file, 0x19, 0x00); 
            break; 
    } 
} 
 
void digimatrix_TurnOnRadio(int file, RADIO_MODE radio_mode)
{
    digimatrix_write_byte_data(file, 0x23, 0x02);

    switch (radio_mode)
    {
        case OFF_MODE:
            // Turns off the Radio
            digimatrix_write_byte_data(file, 0x0f, 0x00);
            break;
        
        case AM_RADIO:
            // Turns on AM Radio;
            digimatrix_write_byte_data(file, 0x0e, 0x00);
            break;
        
        case FM_RADIO:
            // Turns on FM Radio
            digimatrix_write_byte_data(file, 0x0d, 0x00);
            break;
        
        case CD_MODE:
            // Turns on CD mode!!!
            digimatrix_write_byte_data(file, 0x0c, 0x00);
            break;
        
        case HD_MODE:
            // Enables HD mode!!!
            digimatrix_write_byte_data(file, 0x0b, 0x00);
            break;

        default:
            // Turns off the Radio
            digimatrix_write_byte_data(file, 0x0f, 0x00);
            break;
        
    }
}

void digimatrix_DisplayNumbers(int file, char digits[6], LED led, PP pp, DOTS dots, BOOL reinit)
{
    int n;
    
    if (reinit)
    digimatrix_write_byte_data(file, 0x23, 0x02); // Set to digit mode (-- --:--).
    
    // Set up the Dots.
    digimatrix_write_byte_data(file, 0x06, (unsigned char)dots);

    // Set the Play / Pause LEDs
    digimatrix_write_byte_data(file, (unsigned char)pp, 0x01);

    // Now set the mode LEDs
    // If a letter is used as the first digit next, the mode LED will be overridden. 
    if (led != LED_NONE)
    {
        digimatrix_write_byte_data(file, 0x00, (unsigned char) led);
    }

    

    // Set the state of each digit.
    for (n=0; n<6; n++)
    {
        if (digits[n] <= '9' && digits[n] >= '0')
        {
            digimatrix_write_byte_data(file, n, digits[n] - '0');
        }
        else
        {
                switch (digits[n])
                {
                case 'a':
                    digimatrix_write_byte_data(file, n, 0x8b);
                    break;
                case 'A':
                    digimatrix_write_byte_data(file, n, 0x0b);
                    break;

                case 'f':
                    digimatrix_write_byte_data(file, n, 0x8a);
                    break;
                case 'F':
                    digimatrix_write_byte_data(file, n, 0x0a);
                    break;

                case 'c':
                case 'd':
                    digimatrix_write_byte_data(file, n, 0x8e);
                    break;
                case 'C':
                case 'D':
                    digimatrix_write_byte_data(file, n, 0x0e);
                    break;
                
                case 'p':
                    digimatrix_write_byte_data(file, n, 0x8d);
                    break;
                case 'P':
                    digimatrix_write_byte_data(file, n, 0x0d);
                    break;
                
                case 'm':
                case 'M':
                    digimatrix_write_byte_data(file, n, 0x0c);
                    break;
                
		case '-':
		    if (n == 1)
                    	digimatrix_write_byte_data(file, n, 0x0d);
		    else
                    	digimatrix_write_byte_data(file, n, 0x0c);
                    break;

                default:
                    digimatrix_write_byte_data(file, n, 0x0f);
                    break;
                }
        }
    }
    
}

void digimatrix_DisplayTemp(int file, int temp)
{
    char temp_dec = 0;
    // Get the temperature from the IT87


    if (ioperm(0x295, 2, 1) == -1)
    {
        printf("Error getting access\n");
    }
    else
    {
        outb(IT8705_TEMP_REG, 0x295);
        temp = inb(0x296);
    }

    digimatrix_write_byte_data(file, 0x23, 0x02); // Set to digit mode (incase the clock is running)
   
    //digimatrix_write_byte_data(file, 0x0f, 0x01);
    //digimatrix_write_byte_data(file, 0x0f, 0x01);
    //digimatrix_write_byte_data(file, 0x0f, 0x01);

    temp &= 0xFF;
    temp_dec = ((temp / 10) << 4) & 0xf0; 
    temp_dec |= (temp % 10) & 0x0f;
    digimatrix_write_byte_data(file, 0x22, temp_dec);
    //digimatrix_write_byte_data(file, 0x0f, 0x01);
    //digimatrix_write_byte_data(file, 0x0f, 0x01);
}

/* Note: Returns hexified version of temperature: 
         Temp = 34 DegC, return will be 0x34 */

char digimatrix_GetTemp()
{
    int temp;
    char temp_dec = 0;
    // Get the temperature from the IT87


    if (ioperm(0x295, 2, 1) == -1)
    {
        printf("Error getting access\n");
    }
    else
    {
        outb(IT8705_TEMP_REG, 0x295);
        temp = inb(0x296);
    }

    temp &= 0xFF;
    temp_dec = ((temp / 10) << 4) & 0xf0; 
    temp_dec |= (temp % 10) & 0x0f;

    return (temp_dec);
}

void digimatrix_UpdateTemp(int file, int temp)
{
    printf("Temperature = %d\n\r", digimatrix_GetTemp());
    digimatrix_write_byte_data(file, 0x22, digimatrix_GetTemp());
}

static int digimatrix_I2CInit(int i2cbus)
{
  int address = AUDIODJ_I2C_ADDRESS;
  int e1, e2, e3, file;
  char filename1[20];
  char filename2[20];
  char filename3[20];
  char *filename;
  long funcs;
  
  if ((i2cbus < 0) || (i2cbus > 0x3f)) {
    fprintf(stderr,"Error: I2CBUS argument invalid!\n");
    return -1;
  }

  e1 = 0;
/*
 * Try all three variants and give the correct error message
 * upon failure
 */

  sprintf(filename1,"/dev/i2c-%d",i2cbus);
  sprintf(filename2,"/dev/i2c%d",i2cbus);
  sprintf(filename3,"/dev/i2c/%d",i2cbus);
  if ((file = open(filename1,O_RDWR)) < 0) {
    e1 = errno;
    if ((file = open(filename2,O_RDWR)) < 0) {
      e2 = errno;
      if ((file = open(filename3,O_RDWR)) < 0) {
        e3 = errno;
        if(e1 == ENOENT && e2 == ENOENT && e3 == ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s', `%s', or `%s': %s\n",
                     filename1,filename2,filename3,strerror(ENOENT));
        }
        if (e1 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename1,strerror(e1));
          if(e1 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        if (e2 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename2,strerror(e2));
          if(e2 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        if (e3 != ENOENT) {
          fprintf(stderr,"Error: Could not open file `%s' : %s\n",
                     filename3,strerror(e3));
          if(e3 == EACCES)
            fprintf(stderr,"Run as root?\n");
        }
        exit(1);
      } else {
         filename = filename3;
      }
    } else {
       filename = filename2;
    }
  } else {
    filename = filename1;
  }
  
  /* check adapter functionality */
  if (ioctl(file,I2C_FUNCS,&funcs) < 0) {
    fprintf(stderr,
            "Error: Could not get the adapter functionality matrix: %s\n",
            strerror(errno));
    exit(1);
  }

        if (! (funcs &
            (I2C_FUNC_SMBUS_WRITE_BYTE_DATA | I2C_FUNC_SMBUS_READ_BYTE_DATA))) {
           fprintf(stderr, "Error: Adapter for i2c bus %d", i2cbus);
           fprintf(stderr, " does not have byte write capability\n");
           exit(1);
        }       
  
  /* use FORCE so that we can write registers even when
     a driver is also running */
  if (ioctl(file,I2C_SLAVE_FORCE,address) < 0) {
    fprintf(stderr,"Error: Could not set address to %d: %s\n",address,
            strerror(errno));
    exit(1);
  }

  return (file);
}

int main(int argc, char *argv[])
{
  char digits[7];
  int i2cbus=find_i2c_sis96x();
  if (i2cbus == -1)
	return(-1);//Could not detect SiS96x i2c bus, so exit.
  int file=digimatrix_I2CInit(i2cbus);
  if (file <= 0)
	return(-1);//Could not open required bus, so exit.
  int ch, longindex = 0;

  char buffer[20], *ptr;

  BOOL bUpdateTemp = FALSE;
  BOOL bUpdateTime = FALSE;
  BOOL bUpdateVolume = FALSE;

  const struct tm * nice_time;
  time_t nasty_time;
  char hour, minute;
  char volume;
  long vol_current;
  long vol_min, vol_max;
  getvolrange(&vol_min, &vol_max);
  int vol_step=1;
  
  if (argc == 1)
  {
      printf("\nPlease supply some parameters.\nUse %s -h for list.\n\n", argv[0]);
  }

/*  fprintf(stderr,"  WARNING! This program can confuse your I2C bus, "
          "cause data loss and worse!\n Continue (y/n)?");

    ch = getchar(); 

        if(ch != 'y' && ch != 'Y')
                exit(1);
*/
    for (;;) 
    {
        if ((ch = getopt_long(argc, argv, "r:d:D:c:tTulLmsvh", longopts, &longindex)) == -1) 
 //       if ((ch = getopt_long(argc, argv, "i:r:d:D:c:tTulLmsvh", longopts, &longindex)) == -1) 
        {
            break;
        }

      /*  if ( (bI2CInitialised != TRUE) && (ch!='i' && ch!='h' && ch!='v') )
        {
	    printf("\n*** You must supply -i <n> as the first arguement! ***\n\n");
        }
	 */

        switch (ch) 
        {
/*
        case 'i':
            //printf("\nUsing I2C Bus %s\n", optarg);
            file = digimatrix_I2CInit(atoi(optarg));
            if (file > 0)
            {
                bI2CInitialised = TRUE;
            }
            break;
  */      
        case 'r':
                printf("Setting Radio ");
                if (!strcmp(optarg, "fm"))
                {
                    printf("ON  - FM\n");
                    digimatrix_TurnOnRadio(file, FM_RADIO);            
                }
                else if (!strcmp(optarg, "am"))
                {
                    printf("ON - AM\n");
                    digimatrix_TurnOnRadio(file, AM_RADIO);            
                }
                else
                {
                    printf("OFF \n");
                    digimatrix_TurnOnRadio(file, OFF_MODE);            
                }
            break;

        case 'c': 
                printf("Controlling Radio: "); 
                if (!strcmp(optarg, "seek-")) { 
                    printf("Seeking down\n"); 
                    digimatrix_Control(file, SEEK_DOWN); 
                } 
                else if (!strcmp(optarg, "seek+")) { 
                    printf("Seeking up\n"); 
                    digimatrix_Control(file, SEEK_UP); 
                } 
                else if (!strcmp(optarg, "fine-")) { 
                    printf("Fine tuning down\n"); 
                    digimatrix_Control(file, FINE_TUNE_DOWN); 
                } 
                else if (!strcmp(optarg, "fine+")) { 
                    printf("Fine tuning up\n"); 
                    digimatrix_Control(file, FINE_TUNE_UP); 
                } 
                else if (!strcmp(optarg, "ch-")) { 
                    printf("Previous preset\n"); 
                    digimatrix_Control(file, CHANNEL_PREV); 
                } 
                else if (!strcmp(optarg, "ch+")) { 
                    printf("Next preset\n"); 
                    digimatrix_Control(file, CHANNEL_NEXT); 
                } 
                else if (!strcmp(optarg, "prog")) { 
                    printf("Program preset - now select preset with ch+/-\n"); 
                    digimatrix_Control(file, STORE_START); 
                } 
                else if (!strcmp(optarg, "cancel")) { 
                    printf("Program cancelled\n"); 
                    digimatrix_Control(file, STORE_CANCEL); 
                } 
                else if (!strcmp(optarg, "store")) { 
                    printf("Preset stored\n"); 
                    digimatrix_Control(file, STORE_COMMIT); 
                } 
            break; 
 
            
        case 'd':
        case 'D':
	    {
	        LED led = LED_NONE;
		PP pp = PP_NONE;
		DOTS dots = DOTS_NONE;
		
                strncpy(digits, optarg, 6);
                
		printf("Args = %s", optarg);
                printf("\nSetting display to: %s ", digits);
    	        if (strstr(optarg, " play"))
   	        {
		    pp = PP_PLAY;
		    printf(" play");
                }
    	        else if (strstr(optarg, " pause"))
   	        {
		    pp = PP_PAUSE;
		    printf(" pause");
                }
		
    	        if (strstr(optarg, " am"))
   	        {
		    led = LED_AM;
		    printf(" am");
                }
    	        else if (strstr(optarg, " fm"))
   	        {
		    led = LED_FM;
		    printf(" fm");
                }
    	        else if (strstr(optarg, " cd"))
   	        {
		    led = LED_CD;
		    printf(" cd");
                }
    	        else if (strstr(optarg, " hd"))
   	        {
		    led = LED_HD;
		    printf(" hd");
                }


    	        if (strstr(optarg, "both"))
   	        {
		    dots = DOTS_BOTH;
		    printf(" :");
                }
    	        else if (strstr(optarg, "middle"))
   	        {
		    dots = DOTS_MIDDLE;
		    printf(" -");
                }
    	        else if (strstr(optarg, "bottom"))
   	        {
		    dots = DOTS_BOTTOM;
		    printf(" .");
                }
		printf("\n");
		if (ch == 'D')
                digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)1);
		else
                digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
	    }
            break;
            
        case 't':
            printf("Displaying Temperature.\n");
            digimatrix_DisplayTemp(file, 99);
	    break;
	    
        case 'T':
	    bUpdateTime = TRUE;
	    bUpdateTemp = TRUE;
            break;
            
        case 'u':
            printf("Run Temperature Display in update mode.\n");
            digimatrix_DisplayTemp(file, 99);
            bUpdateTemp = TRUE;
            break;
            
        case 'l':
            printf("display volume control. \nVolume Change: ");
            //digimatrix_write_byte(file, 0x24);
            //volume = digimatrix_read_byte(file);
            //printf("%d\n", volume);
	    bUpdateVolume = TRUE;
	    break;

	case 'L':
            digimatrix_write_byte(file, 0x24);
            volume = digimatrix_read_byte(file);
            printf("%d\n", volume);
            break;

        case 'm':
            printf("Entering Interactive mode. Press enter to exit\n");
            printf("Please enter \"REG VALUE\"\n");
            printf("Where REG and VALUE are decimal integers.\n");
            while (buffer[0] != '\n')
            {
                if (fgets(buffer, 20, stdin)) // Get at most 10 bytes from user.
                {
                    if (atoi(buffer) > 86)
                    {
                        printf("I'm not letting you write there - the I2C bus will lock up!\n");
                    }
                    else
                    {
                        ptr = strstr(buffer, " "); // Find the start of the data value
                        if (ptr)
                        {
                            printf("0x%X = 0x%X ", atoi(buffer), atoi(ptr));
                            digimatrix_write_byte_data(file, atoi(buffer), atoi(ptr));
                            printf("Read Back: 0x%0X\n", digimatrix_read_byte(file));
                        }
                    }   
                }
            }

            break;

        case 's':
            time(&nasty_time);
            nice_time = localtime(&nasty_time);
            hour = ((nice_time->tm_hour / 10) << 4) | (nice_time->tm_hour % 10);
            minute = ((nice_time->tm_min / 10) << 4) | (nice_time->tm_min % 10);
 
            printf("Setting and displaying time (%X:%X).\n", hour, minute);
            digimatrix_write_byte_data(file, 30, hour);
            digimatrix_write_byte_data(file, 31, minute);
            digimatrix_write_byte_data(file, 41, 0x01);
            break;

        case 'v':
            printf("ASUS DigiMatrix Panel Control Version %s\n", DM_VERSION);
            break;

        case 'h':
            help(argv[0]);
            break;

        default:
            help(argv[0]);
            break;
        }
    }

    if (bUpdateTemp || bUpdateTime || bUpdateVolume)
    {
        unsigned int temp_counter = 0;
        time_t current_time;
        struct tm *time_date;
        LED led = LED_NONE;
        PP pp = PP_NONE;
        static DOTS dots = DOTS_NONE;
        char ascii_time[5];
	//soundCtrl SoundControl("/dev/mixer");

        while (1)
        {
            if (bUpdateTemp || bUpdateTime)
            {
                if (temp_counter >= (TEMPERATURE_UPDATE_INTERVAL_MS / VOLUME_UPDATE_INTERVAL_MS))
                {

                    // Make the dots flash. 
                    if (dots == DOTS_NONE)
		        dots = DOTS_BOTH;
		    else
		        dots = DOTS_NONE;
			
                    temp_counter = 0;
		    
		    time(&current_time);
		    time_date = localtime(&current_time);
		    strftime(ascii_time, 5, "%H%M", time_date);
		    
		    sprintf(digits, "%02X%s", digimatrix_GetTemp(), ascii_time);
                    digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
                }
            }
            else if (bUpdateTemp)
            {
                if (temp_counter >= (TEMPERATURE_UPDATE_INTERVAL_MS / VOLUME_UPDATE_INTERVAL_MS))
                {
                    temp_counter = 0;
                    digimatrix_UpdateTemp(file, 99);
                    //fprintf(stderr, "\rUpdated Temperature.   ");
                }
            }

	    if (bUpdateVolume)
	    {
                digimatrix_write_byte(file, 0x24);
                volume = digimatrix_read_byte(file);
		//printf("Volume: %d\n\r", volume);
		
		if (volume != -120 && volume != 0)
		{
		    for (temp_counter=12; temp_counter > 0; temp_counter--)
		    {
				getvolume(&vol_current);
				if (volume > 0) //use vol_max-1 so never 100 (3digits)
					if (vol_current+vol_step < vol_max-1)
						vol_current=vol_current+vol_step;
					else
						vol_current=vol_max-1;
				else if (volume < 0){
					if (vol_current-vol_step > vol_min)
						vol_current=vol_current-vol_step;
					else
						vol_current=vol_min;
					}
				setvolume(vol_current);
			
				dots = DOTS_NONE;
			    sprintf(digits, "%02ld    ",
					(100*(vol_current-vol_min))/(vol_max-vol_min));
		    	digimatrix_DisplayNumbers(file, digits, led, pp, dots, (BOOL)0);
                        
				usleep(VOLUME_UPDATE_INTERVAL_MS * 1000); // Delay of 500ms
                        digimatrix_write_byte(file, 0x24);
                        volume = digimatrix_read_byte(file);
		    }
		}
		
		
	    }

            usleep(VOLUME_UPDATE_INTERVAL_MS * 1000); // Delay of 500ms
            temp_counter++;
        }
    }
    
    exit (0);
}
